@@ -63,6 +63,10 @@ typedef struct
   guint state;
 
   gboolean autohide;
+  gboolean autohide_missing;
+
+  int size;
+  GtkOrientation orientation;
   
   GdkPixbuf *pb[7];
 
@@ -103,11 +107,11 @@ load_and_scale(const guint8 *data, int dstw, int dsth)
 }
 
 static void
-wavelan_load_pixbufs(t_wavelan *wavelan, int size)
+wavelan_load_pixbufs(t_wavelan *wavelan)
 {
-  int n;
+  int n, w, h;
 
-  TRACE ("Entered wavelan_load_pixbufs, size = %d", size);
+  TRACE ("Entered wavelan_load_pixbufs, size = %d", wavelan->size);
 
   /*
    * free old pixbufs first
@@ -117,16 +121,28 @@ wavelan_load_pixbufs(t_wavelan *wavelan, int size)
       g_object_unref(G_OBJECT(wavelan->pb[n]));
   }
 
+  /*
+   * Determine dimension
+   */
+  if (wavelan->orientation == GTK_ORIENTATION_HORIZONTAL) {
+    w = -1;
+    h = wavelan->size;
+  }
+  else {
+    w = wavelan->size;
+    h = -1;
+  }
+
   /*
    * Load and scale pixbufs
    */
-  wavelan->pb[0] = load_and_scale(error_icon_data, size, size);
-  wavelan->pb[1] = load_and_scale(link0_icon_data, size, size);
-  wavelan->pb[2] = load_and_scale(link1_icon_data, size, size);
-  wavelan->pb[3] = load_and_scale(link2_icon_data, size, size);
-  wavelan->pb[4] = load_and_scale(link3_icon_data, size, size);
-  wavelan->pb[5] = load_and_scale(link4_icon_data, size, size);
-  wavelan->pb[6] = load_and_scale(link5_icon_data, size, size);
+  wavelan->pb[0] = load_and_scale(error_icon_data, w, h);
+  wavelan->pb[1] = load_and_scale(link0_icon_data, w, h);
+  wavelan->pb[2] = load_and_scale(link1_icon_data, w, h);
+  wavelan->pb[3] = load_and_scale(link2_icon_data, w, h);
+  wavelan->pb[4] = load_and_scale(link3_icon_data, w, h);
+  wavelan->pb[5] = load_and_scale(link4_icon_data, w, h);
+  wavelan->pb[6] = load_and_scale(link5_icon_data, w, h);
 }
 
 static void
@@ -146,6 +162,8 @@ wavelan_set_state(t_wavelan *wavelan, guint state)
 
   if (wavelan->autohide && state == STATE_LINK0)
     gtk_widget_hide(wavelan->ebox);
+  else if (wavelan->autohide_missing && state == STATE_ERROR)
+    gtk_widget_hide(wavelan->ebox);
   else
     gtk_widget_show(wavelan->ebox);
 }
@@ -258,6 +276,7 @@ wavelan_read_config(XfcePanelPlugin *plugin, t_wavelan *wavelan)
       }
       
       wavelan->autohide = xfce_rc_read_bool_entry (rc, "Autohide", FALSE);
+      wavelan->autohide_missing = xfce_rc_read_bool_entry(rc, "AutohideMissing", FALSE);
     }
   }
   
@@ -276,14 +295,15 @@ wavelan_new(XfcePanelPlugin *plugin)
 	wavelan = g_new0(t_wavelan, 1);
 
   wavelan->autohide = FALSE;
+  wavelan->autohide_missing = FALSE;
 
   wavelan->plugin = plugin;
   
-  size = xfce_panel_plugin_get_size (plugin);
+  wavelan->size = xfce_panel_plugin_get_size (plugin);
   screen_position = xfce_panel_plugin_get_screen_position (plugin);
+  wavelan->orientation = xfce_panel_plugin_get_orientation (plugin);
  
   wavelan->ebox = gtk_event_box_new();
-  gtk_widget_show (wavelan->ebox);
   gtk_container_add (GTK_CONTAINER (plugin), wavelan->ebox);
   
   if (xfce_screen_position_is_horizontal (screen_position))
@@ -297,7 +317,7 @@ wavelan_new(XfcePanelPlugin *plugin)
   /*wavelan->button = xfce_create_panel_button ();
   gtk_widget_show (wavelan->button);
 	gtk_box_pack_start(GTK_BOX (wavelan->box), wavelan->button, TRUE, TRUE, 0);*/
-  gtk_widget_set_size_request (wavelan->box, size, size);
+  gtk_widget_set_size_request (wavelan->box, -1, -1);
   
   xfce_panel_plugin_add_action_widget (plugin, wavelan->box);
 
@@ -311,9 +331,13 @@ wavelan_new(XfcePanelPlugin *plugin)
   g_object_ref (wavelan->tooltips);
   gtk_object_sink (GTK_OBJECT (wavelan->tooltips));
 
-  wavelan_load_pixbufs(wavelan, size);
+  wavelan_load_pixbufs(wavelan);
   
   wavelan_read_config(plugin, wavelan);
+
+  wavelan_set_state(wavelan, wavelan->state);
+
+  gtk_widget_show_all(wavelan->ebox);
   
   return(wavelan);
 }
@@ -371,6 +395,7 @@ wavelan_write_config(XfcePanelPlugin *plugin, t_wavelan *wavelan)
     xfce_rc_write_entry (rc, "Interface", wavelan->interface);
   }
   xfce_rc_write_bool_entry (rc, "Autohide", wavelan->autohide);
+  xfce_rc_write_bool_entry (rc, "AutohideMissing", wavelan->autohide_missing);
 
   xfce_rc_close(rc);
   
@@ -379,14 +404,16 @@ wavelan_write_config(XfcePanelPlugin *plugin, t_wavelan *wavelan)
 static void
 wavelan_set_orientation(t_wavelan *wavelan, GtkOrientation orientation)
 {
+  wavelan->orientation = orientation;
   xfce_hvbox_set_orientation (XFCE_HVBOX (wavelan->box), orientation);
 }
 
 static void
 wavelan_set_size(t_wavelan *wavelan, int size)
 {
-  gtk_widget_set_size_request (wavelan->box, size, size);
-  wavelan_load_pixbufs(wavelan, size);  
+  wavelan->size = size;
+  wavelan_load_pixbufs(wavelan);
+  gtk_widget_set_size_request (wavelan->box, -1, -1);
 }
 
 /* interface changed callback */
@@ -408,6 +435,15 @@ wavelan_autohide_changed(GtkToggleButton *button, t_wavelan *wavelan)
   wavelan_set_state(wavelan, wavelan->state);
 }
 
+/* autohide on missing callback */
+static void 
+wavelan_autohide_missing_changed(GtkToggleButton *button, t_wavelan *wavelan)
+{
+  TRACE ("Entered wavelan_autohide_missing_changed");
+  wavelan->autohide_missing = gtk_toggle_button_get_active(button);
+  wavelan_set_state(wavelan, wavelan->state);
+}
+
 /* query installed devices */
 static GList*
 wavelan_query_interfaces (void)
@@ -453,7 +489,8 @@ wavelan_dialog_response (GtkWidget *dlg, int response, t_wavelan *wavelan)
 static void
 wavelan_create_options (XfcePanelPlugin *plugin, t_wavelan *wavelan)
 {
-  GtkWidget *dlg, *hbox, *label, *interface, *vbox, *autohide, *header;
+  GtkWidget *dlg, *hbox, *label, *interface, *vbox, *autohide;
+  GtkWidget *autohide_missing, *header, *warn_label;
   GtkWidget *combo;
   GList     *interfaces, *lp;
 
@@ -520,6 +557,25 @@ wavelan_create_options (XfcePanelPlugin *plugin, t_wavelan *wavelan)
   gtk_widget_show(autohide);
   gtk_box_pack_start(GTK_BOX(hbox), autohide, TRUE, TRUE, 1);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 1);
+  
+  hbox = gtk_hbox_new(FALSE, 2);
+  gtk_widget_show(hbox);
+  autohide_missing = gtk_check_button_new_with_label(_("Autohide when no hardware present"));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(autohide_missing), 
+      wavelan->autohide_missing);
+  g_signal_connect(autohide_missing, "toggled", 
+      G_CALLBACK(wavelan_autohide_missing_changed), wavelan);
+  gtk_widget_show(autohide_missing);
+  gtk_box_pack_start(GTK_BOX(hbox), autohide_missing, TRUE, TRUE, 1);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 1);
+
+  hbox = gtk_hbox_new(FALSE, 2);
+  gtk_widget_show(hbox);
+  warn_label = gtk_label_new(_("Note: This will make it difficult to remove or configure the plugin if there is no device detected."));
+  gtk_label_set_line_wrap(GTK_LABEL(warn_label), TRUE);
+  gtk_widget_show(warn_label);
+  gtk_box_pack_start(GTK_BOX(hbox), warn_label, TRUE, TRUE, 1);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 1);
 
   for (lp = interfaces; lp != NULL; lp = lp ->next)
     g_free (lp->data);
